@@ -10,18 +10,20 @@ from ._core import (
     list_pods,
     logs_streaming,
 )
-from ._types import Job
-from ._backend import make_job
+from ._types import Job, CronJob
+from ._backend import make_job, make_cronjob
 
 
-__version__ = "0.3.2"
+__version__ = "0.4.1"
 
 __all__ = [
     "__version__",
     "Job",
+    "CronJob",
     "list_jobs",
     "submit_job",
     "make_job",
+    "make_cronjob",
     "configure",
     "show_job",
     "logs",
